unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  PegtopFormulas, StdCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    Button1: TButton;
    Memo1: TMemo;
    Edit1: TEdit;
    Image1: TImage;
    Formula: TPegtopFormula;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.Button1Click(Sender: TObject);
var
  R: Double;
  X, Y, H: Integer;
  Heights: array[0..299] of Integer;
begin
  Formula.Expression := Edit1.Text;
  try
    // force formula to be compiled:
    Formula.Compile;
    // (if we don't it automatically gets compiled when needed,
    // but we want to catch exceptions here)
  except
    // clear image:
    Image1.Picture.Bitmap.Canvas.FillRect(Rect(0, 0, Image1.Picture.Bitmap.Width, Image1.Picture.Bitmap.Height));
    raise; // raise exception again
  end;

  // show compiled asm code:
  Memo1.Lines := Formula.CodeLines;
  Memo1.Lines.Add('(' + IntToStr(Formula.CodeSize) + ' bytes)');

  // draw image:
  for X := Low(Heights) to High(Heights) do Heights[X] := Image1.Picture.Bitmap.Height;
  Image1.Picture.Bitmap.Canvas.FillRect(Rect(0, 0, Image1.Picture.Bitmap.Width, Image1.Picture.Bitmap.Height));
  for Y := 0 to 127 do begin
    // set 'y' value:
    Formula.SetArg('y', Y / 128 * 4 - 2);
    if ((Y mod 16) = 0) then
      // some lines are white
      Image1.Picture.Bitmap.Canvas.Pen.Color := (255 - Y * 2) * $010101
    else
      // other lines are blue
      Image1.Picture.Bitmap.Canvas.Pen.Color := (255 - Y * 2) * $010000;
    for X := Low(Heights) to High(Heights) do begin
      // set 'x' value:
      Formula.SetArg(0, X/Image1.Picture.Bitmap.Width*4.0 - 2.0);
      // (SetArg is overloaded, it's faster to use index 0 than to use name 'x')
      try
        R := Formula.Eval*128/4;
        if IsInfiniteOrNaN(R) then begin
          if IsPosInfinite(R) then
            H := -1 // top of image
          else if IsNegInfinite(R) then
            H := Image1.Picture.Bitmap.Height // bottom of image
          else
            H := Image1.Picture.Bitmap.Height - Y - 100; // zero
        end
        else begin
          H := Image1.Picture.Bitmap.Height - Y - Round(R) - 100;
        end;
      except
        H := Image1.Picture.Bitmap.Height - Y - 100; // zero
      end;
      if H < Heights[X] then begin
        Image1.Picture.Bitmap.Canvas.MoveTo(X, H);
        Image1.Picture.Bitmap.Canvas.LineTo(X, Heights[X]);
        Heights[X] := H;
      end;
    end
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Image1.Picture.Bitmap.Canvas.Brush.Style := bsSolid;
  Image1.Picture.Bitmap.Canvas.Brush.Color := clBlack;
  Image1.Picture.Bitmap.Width := Image1.Width;
  Image1.Picture.Bitmap.Height := Image1.Height;
end;

end.
